package clasesJava;

import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Properties;
import java.sql.DriverManager;

public class AccesoBD {
	
	/**
	 * Maps a properties file to a Properties object
	 * @param file Properties file name
	 * @return Properties Properties object with the info from the file 
	 * @throws IOException Occurs if the file is not found
	 */
	private static Properties loadParams( String file )
		throws IOException {
		Properties prop = new Properties();
		prop.load(new FileInputStream (file));
		return prop;
	}

	/**
	 * Obtains a connection to the database, reading the properties
	 * file (bd.properties) previously.
	 * @return DB connection or null if a problem occurred when trying to connect.
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 * @throws SQLException 
	 */
	public static Connection getConnection() throws IOException, ClassNotFoundException, SQLException {
		Connection con = null;
		Class.forName("org.postgresql.Driver");
		HashMap <String, String> parameters = loadDBParams();
	    String dbURL = "jdbc:postgresql:"+parameters.get("dbname");
	    try{
	    	con = DriverManager.getConnection(dbURL,parameters.get("user"),parameters.get("passwd"));
	    }catch (SQLException e) {
			e.printStackTrace();
		}
	    return con;
	}

	/**
	 * Closes the DB connection.
	 * @param con Connection to the DB.
	 */
	public static void closeConnection( Connection con ) {
		try {
			con.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * Load the properties file for the database and assigns them to 
	 * the class attributes
	 * @throws IOException Properties file could not be found.
	 */
	private static HashMap<String,String> loadDBParams()
		throws IOException {
		// Get the database configuration
		Properties prop = null;
		HashMap<String,String> parameters = new HashMap<String, String>(6);
		try {
			prop = loadParams( "C:/TFC/bd.properties" );
			parameters.put("host", prop.getProperty( "host" ));
			parameters.put("port", prop.getProperty( "port" ));
			parameters.put("dbname", prop.getProperty( "dbname"));
			parameters.put("params", prop.getProperty( "params"));
			parameters.put("user", prop.getProperty( "user"));
			parameters.put("passwd",prop.getProperty( "passwd"));
		}
		catch ( IOException e ) {
			System.err.println( "ERROR: bd.properties file could not be found");
			throw e;
		}
		return parameters;
	}

}
