package clasesJava;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Security;
import java.security.SignatureException;
import java.security.cert.CRLException;
import java.security.cert.CertificateException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Date;
import java.util.Properties;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.asn1.x509.CRLReason;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.x509.X509V2CRLGenerator;
import org.bouncycastle.x509.extension.AuthorityKeyIdentifierStructure;

public class CRLBC {
	
	/**
	 * Constructor per defecte
	 */
	public CRLBC(){
	}
	
	/**
	 * Mtode que crea una CRL
	 * @return crlBC
	 */
	public X509CRL creaCRL(){
		
		X509CRL crlBC = null;
		
		// Carrega les properties
		try {
			Properties prop = new Properties();
			prop = loadParams( "C:/TFC/rutas.properties" );
			String r1=prop.getProperty("pswd2");
			String r2=prop.getProperty("magatzem");
			String r3=prop.getProperty("clauCApriv");
			
			// Afegir provider BC
			Security.addProvider(new BouncyCastleProvider());
			
			KeyFactory keyFactoryRSA;
			try {
				keyFactoryRSA = KeyFactory.getInstance("RSA", "BC");
				
				// Recupera el certificat de la CA
				String nom2 = r2;
				char[] password = r1.toCharArray();
				KeyStore magatzem = KeyStore.getInstance("PKCS12", "BC");
				FileInputStream in = new FileInputStream(nom2);
				magatzem.load(in, password);
				in.close();
				X509Certificate CAcert = (X509Certificate) magatzem.getCertificate("Criptotal");
				
				// Recupera la clau privada de la CA
				String nom3=r3;
				byte[] bufferPrivCA = new byte[5000];
				in = new FileInputStream(nom3);
				in.read(bufferPrivCA, 0, 5000);
				in.close();
				PKCS8EncodedKeySpec clauPrivadaSpec = new PKCS8EncodedKeySpec(bufferPrivCA);
				PrivateKey CAKey = keyFactoryRSA.generatePrivate(clauPrivadaSpec);
				
				Date now = new Date();
				
				// Genera la CRL
				X509V2CRLGenerator crlGen = new X509V2CRLGenerator();
				crlGen.setIssuerDN(new X500Principal("CN=Criptotal"));
				crlGen.setThisUpdate(now);
				crlGen.setSignatureAlgorithm(CAcert.getSigAlgName());
				crlGen.addExtension(X509Extensions.AuthorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(CAcert));
				crlBC = crlGen.generateX509CRL(CAKey, "BC");
				
			} catch (NoSuchAlgorithmException e) {
				System.out.println("No existeix aquest algorisme");
				e.printStackTrace();
			} catch (NoSuchProviderException e) {
				System.out.println("No existeix aquest proveidor de seguretat");
				e.printStackTrace();
			}catch (FileNotFoundException e) {
				System.out.println("El fitxer no existeix");
			} catch (IOException e) {
				System.out.println("Problemes de lectura/escriptura");
			} catch (KeyStoreException e) {
				System.out.println("KeyStore no vlid");
				e.printStackTrace();
			} catch (CertificateException e) {
				System.out.println("Certificat no vlid");
				e.printStackTrace();
			} catch (InvalidKeySpecException e) {
				System.out.println("KeySpec no vlid");
				e.printStackTrace();
			} catch (InvalidKeyException e) {
				System.out.println("Clau no vlida");
				e.printStackTrace();
			} catch (SignatureException e) {
				System.out.println("Signatura no vlida");
				e.printStackTrace();
			}
		
		} catch (IOException e1) {
			System.out.println("Excepci d'entrada-sortida");
			e1.printStackTrace();
		}
		
		return crlBC;
	}
	
	/**
	 * Mtode que afegeix un certificat a la CRL
	 * @param serial El nombre de serie del certificat
	 */
	
	public X509CRL afegirCert(int serial, X509CRL crlBC){
		
		X509CRL nuevaLista = null;
		
		// Carrega les properties
		try {
			Properties prop = new Properties();
			prop = loadParams( "C:/TFC/rutas.properties" );
			String r1=prop.getProperty("pswd2");
			String r2=prop.getProperty("magatzem");
			String r3=prop.getProperty("clauCApriv");
					
			KeyFactory keyFactoryRSA;
			try {
				
				// Afegir provider BC
				Security.addProvider(new BouncyCastleProvider());

				keyFactoryRSA = KeyFactory.getInstance("RSA", "BC");
				
				// Recupera el certificat de la CA
				String nom2 = r2;
				char[] password = r1.toCharArray();
				KeyStore magatzem = KeyStore.getInstance("PKCS12", "BC");
				FileInputStream in = new FileInputStream(nom2);
				magatzem.load(in, password);
				in.close();
				X509Certificate CAcert = (X509Certificate) magatzem.getCertificate("Criptotal");
				
				// Recupera la clau privada de la CA
				String nom3=r3;
				byte[] bufferPrivCA = new byte[5000];
				in = new FileInputStream(nom3);
				in.read(bufferPrivCA, 0, 5000);
				in.close();
				PKCS8EncodedKeySpec clauPrivadaSpec = new PKCS8EncodedKeySpec(bufferPrivCA);
				PrivateKey CAKey = keyFactoryRSA.generatePrivate(clauPrivadaSpec);
				
				// Genera la CRL i afegeix el certificat revocat
				BigInteger serialNumber = BigInteger.valueOf(serial);
				Date now = new Date();
				X509V2CRLGenerator crlGen = new X509V2CRLGenerator();
				crlGen.setIssuerDN(new X500Principal("CN=Criptotal"));
				crlGen.setThisUpdate(now);
				crlGen.setSignatureAlgorithm(CAcert.getSigAlgName());
				crlGen.addExtension(X509Extensions.AuthorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(CAcert));
				crlGen.addCRL(crlBC);
				crlGen.addCRLEntry(serialNumber, now, CRLReason.privilegeWithdrawn);
				nuevaLista = crlGen.generateX509CRL(CAKey, "BC");
				
			} catch (CRLException e){
				System.out.println("Problemes amb la CRL");
				e.printStackTrace();
			} catch (NoSuchAlgorithmException e) {
				System.out.println("No existeix aquest algorisme");
				e.printStackTrace();
			} catch (NoSuchProviderException e) {
				System.out.println("No existeix aquest proveidor de seguretat");
				e.printStackTrace();
			}catch (FileNotFoundException e) {
				System.out.println("El fitxer no existeix");
			} catch (IOException e) {
				System.out.println("Problemes de lectura/escriptura");
			} catch (KeyStoreException e) {
				System.out.println("KeyStore no vlid");
				e.printStackTrace();
			} catch (CertificateException e) {
				System.out.println("Certificat no vlid");
				e.printStackTrace();
			} catch (InvalidKeySpecException e) {
				System.out.println("KeySpec no vlid");
				e.printStackTrace();
			} catch (InvalidKeyException e) {
				System.out.println("Clau no vlida");
				e.printStackTrace();
			} catch (SignatureException e) {
				System.out.println("Signatura no vlida");
				e.printStackTrace();
			}
		
		} catch (IOException e1) {
			System.out.println("Excepci d'entrada-sortida");
			e1.printStackTrace();
		}
		
		return nuevaLista;

	}
	
	
		
		/**
		 * Maps a properties file to a Properties object
		 * @param file Properties file name
		 * @return Properties Properties object with the info from the file 
		 * @throws IOException Occurs if the file is not found
		 */
		private static Properties loadParams( String file )
			throws IOException {
			Properties prop = new Properties();
			prop.load(new FileInputStream (file));
			return prop;
		}

}
