package clasesJava;

import java.io.IOException;
import java.security.PublicKey;
import java.security.Security;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class InsertaCerti {
	
	private static final String PK_ERROR = "23505";
	private static final String FK_ERROR = "23503";
	
	/**
	 * Default constructor.
	 */
	public InsertaCerti() {
	}
	
	/**
	 * Inserts a cert to the database
	 * @param cert
	 * @throws SQLException 
	 */
	public int insercioBD (Connection con, Cert cert) throws SQLException
		 {
		
		String sql1 = "SELECT COUNT(*)AS nombre FROM certificados";
		String sql2 = "INSERT INTO certificados(numero,nom,cognom1,cognom2,dataInici,dataFi,valido) VALUES (?,?,?,?,?,?,?);";

		PreparedStatement pst = con.prepareStatement(sql1);
		ResultSet rs = pst.executeQuery();
		rs.next();
		int nombre = rs.getInt(1)+1;
		
		pst = con.prepareStatement(sql2);
		pst.clearParameters();
		pst.setInt(1, nombre);
		pst.setString(2,cert.getNom());
		pst.setString(3,cert.getCognom1());
		pst.setString(4,cert.getCognom2());
		pst.setDate(5,new java.sql.Date (cert.getDataInici().getTime()));
		pst.setDate(6,new java.sql.Date (cert.getDataFi().getTime()));
		pst.setBoolean(7, cert.getValid());
		pst.executeUpdate();
		pst.close();
		
		return nombre;
			
	}
	
	/**
	 * Connects, inserts and disconnetcs.
	 * @param args Method parameters
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 * @throws IOException 
	 */
	public String insertarCert(Cert cert) throws IOException, ClassNotFoundException, SQLException {
		String arxiu="";
		int nombre=0;
		Connection con = null;

		con = AccesoBD.getConnection();
		con.setAutoCommit(false);
		try {
			nombre = insercioBD(con, cert);
			//con.commit();
			arxiu = cridaCerti(nombre,cert);
		}
		catch (SQLException e){
			if(e.getSQLState().equals(FK_ERROR)){
				System.out.println("ERROR: Clau inexistent");
			}else if (e.getSQLState().equals(PK_ERROR)){
				System.out.println("ERROR: Clau duplicada");
			} else System.out.println("Unexpected error: " + e.getMessage());
		}
		catch (Exception e) {
			con.rollback();
			System.out.println("ERROR (" + e.getClass() + "): " + e.getMessage());
		}
		
		if (arxiu==""){
			con.rollback();
		}else{
			con.commit();
		}

		AccesoBD.closeConnection(con);
		
		return arxiu;
	}
	
	/**
	 * Connects, inserts and disconnetcs.
	 * @param args Method parameters
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 * @throws IOException 
	 */
	public String insertarCert(Cert cert, PublicKey clauPubUs) throws IOException, ClassNotFoundException, SQLException {
		String arxiu="";
		int nombre=0;
		Connection con = null;

		con = AccesoBD.getConnection();
		con.setAutoCommit(false);
		try {
			nombre = insercioBD(con, cert);
			//con.commit();
			arxiu = cridaCerti(nombre,cert,clauPubUs);
		}
		catch (SQLException e){
			if(e.getSQLState().equals(FK_ERROR)){
				System.out.println("ERROR: Clau inexistent");
			}else if (e.getSQLState().equals(PK_ERROR)){
				System.out.println("ERROR: Clau duplicada");
			} else System.out.println("Unexpected error: " + e.getMessage());
		}
		catch (Exception e) {
			con.rollback();
			System.out.println("ERROR (" + e.getClass() + "): " + e.getMessage());
		}
		
		if (arxiu==""){
			con.rollback();
		}else{
			con.commit();
		}

		AccesoBD.closeConnection(con);
		
		return arxiu;
	}
	
	public String cridaCerti(int serialNumber, Cert cert){
		String usuari = cert.getNom()+" "+cert.getCognom1()+" "+cert.getCognom2();
		Date dataInici = cert.getDataInici();
		Date dataFi = cert.getDataFi();
		parellClausUsuari pcUS = new parellClausUsuari();
		pcUS.generaClausUsuari(serialNumber);
		certificatUsuari cUS = new certificatUsuari();
		String arxiu = cUS.generaCertificatUsuari(serialNumber, usuari, dataInici, dataFi);
		return arxiu;
	}
	
	public String cridaCerti(int serialNumber, Cert cert, PublicKey clau){
		
		Security.addProvider(new BouncyCastleProvider());
						
		// Creem el certificat
		String usuari = cert.getNom()+" "+cert.getCognom1()+" "+cert.getCognom2();
		Date dataInici = cert.getDataInici();
		Date dataFi = cert.getDataFi();
		certificatUsuari cUS = new certificatUsuari();
		String arxiu = cUS.generaCertificatUsuari2(serialNumber, usuari, dataInici, dataFi, clau);
		return arxiu;
 
	}

}
